/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DPPReader.h"
#include <iostream>

using namespace std;

namespace npp {

DPPReader::DPPReader(int32_t size) :
	_size(0),
	_sizeInBytes(0),
	_memoryBlock(0),
	_nbAdc(0),
	_nbChannels(0),
	_listMode(0),
	_listModeSize(0) {

	init(size);
}

DPPReader::~DPPReader() {
	delete [] _memoryBlock;
}

void DPPReader::init(int32_t size) {
	_size = size;
	_sizeInBytes = size * sizeof(int32_t);
	_memoryBlock = new int32_t[size];
}

bool DPPReader::openFile(std::string const & fileName) {

	_dataFile.open(fileName.c_str(), ios::in | ios::binary);

	if (!_dataFile.is_open()) {
		cerr << "problem reading " << fileName << endl;
		return false;
	}

	_dataFile.seekg(0, ios::end);
	_fileSize = _dataFile.tellg();
	_dataFile.seekg(0, ios::beg);

	// just reading 4 elements for the header
	_dataFile.read((char *)_memoryBlock, 4 * sizeof(int32_t));

	_nbAdc = _memoryBlock[0];
	_nbChannels = _memoryBlock[1];
	_listMode = _memoryBlock[2];
	_listModeSize = _memoryBlock[3];

	return true;
}

int32_t DPPReader::numberOfADCs() const {
	return _nbAdc;
}

int32_t DPPReader::numberOfChannels() const {
	return _nbChannels;
}

int32_t DPPReader::listMode() const {
	return _listMode;
}

int32_t DPPReader::numberOfEvents() const {
	return _listModeSize;
}

int32_t DPPReader::readBlock() {

	// reading next block
	int32_t blockSize = getBlockSizeInBytes();
	_dataFile.read((char *)_memoryBlock, blockSize);

	return (blockSize / 4);
}

int32_t DPPReader::getBlockSizeInBytes() {

	FileSize size = _fileSize - _dataFile.tellg();
	if (size < (FileSize)_sizeInBytes) {
		return size;
	}

	return _sizeInBytes;
}

int32_t * DPPReader::memoryBlock() const {
	return _memoryBlock;
}

}
